@@ -16,6 +16,8 @@ pub enum CallError {
     InvalidTopic = 9,
     MissingCallbacks = 10,
     Unsupported = 11,
+    BadRequest = 12,
+    InternalFailure = 14,
     Unknown = u32::MAX,
 }
 
@@ -34,6 +36,8 @@ impl From<u32> for CallError {
             9 => CallError::InvalidTopic,
             10 => CallError::MissingCallbacks,
             11 => CallError::Unsupported,
+            12 => CallError::BadRequest,
+            14 => CallError::InternalFailure,
             _ => CallError::Unknown,
         }
     }
@@ -54,6 +58,8 @@ impl Into<u32> for CallError {
             CallError::InvalidTopic => 9,
             CallError::MissingCallbacks => 10,
             CallError::Unsupported => 11,
+            CallError::BadRequest => 12,
+            CallError::InternalFailure => 14,
             CallError::Unknown => u32::MAX,
         }
     }
@@ -74,7 +80,7 @@ impl Into<io::Error> for CallError {
             ),
             err => io::Error::new(
                 io::ErrorKind::Other,
-                format!("wapm bus error - {}", err.to_string()).as_str(),
+                format!("wasm bus error - {}", err.to_string()).as_str(),
             ),
         }
     }
@@ -105,6 +111,11 @@ impl fmt::Display for CallError {
             CallError::Unsupported => {
                 write!(f, "this operation is not supported on this platform.")
             }
+            CallError::BadRequest => write!(
+                f,
+                "invalid input was supplied in the call resulting in a bad request."
+            ),
+            CallError::InternalFailure => write!(f, "an internal failure has occured"),
             CallError::Unknown => write!(f, "unknown error."),
         }
     }
