@@ -1,6 +1,8 @@
 #![allow(dead_code)]
 use once_cell::sync::Lazy;
 use serde::*;
+#[allow(unused_imports, dead_code)]
+use std::any::type_name;
 use std::borrow::Cow;
 #[allow(unused_imports)]
 use std::future::Future;
@@ -102,11 +104,30 @@ impl BusEngine {
             if let Some(call) = state.calls.get(&handle) {
                 let call = Arc::clone(call);
                 drop(state);
+                trace!(
+                    "wasm_bus_finish (handle={}, response={} bytes, wapm={}, topic={})",
+                    handle.id,
+                    response.len(),
+                    call.wapm(),
+                    call.topic()
+                );
                 call.data(response);
             } else if let Some(callback) = state.callbacks.get(&handle) {
                 let callback = Arc::clone(callback);
                 drop(state);
+                trace!(
+                    "wasm_bus_finish (handle={}, response={} bytes, topic={})",
+                    handle.id,
+                    response.len(),
+                    callback.topic()
+                );
                 let _ = callback.process(response);
+            } else {
+                trace!(
+                    "wasm_bus_finish (handle={}, response={} bytes, orphaned)",
+                    handle.id,
+                    response.len()
+                );
             }
         };
 
@@ -119,10 +140,24 @@ impl BusEngine {
 
     pub fn error(handle: CallHandle, err: CallError) {
         {
-            let mut state = BusEngine::write();
-            if let Some(call) = state.calls.remove(&handle) {
+            let state = BusEngine::read();
+            if let Some(call) = state.calls.get(&handle) {
+                let call = Arc::clone(call);
                 drop(state);
+                trace!(
+                    "wasm_bus_err (handle={}, error={}, wapm={}, topic={})",
+                    handle.id,
+                    err,
+                    call.wapm(),
+                    call.topic()
+                );
                 call.error(err);
+            } else {
+                trace!(
+                    "wasm_bus_err (handle={}, error={}, orphaned)",
+                    handle.id,
+                    err
+                );
             }
         }
 
@@ -141,19 +176,45 @@ impl BusEngine {
         wakers.insert(handle.clone(), waker);
     }
 
-    pub fn remove(handle: &CallHandle) {
-        if let Some(mut state) = BusEngine::try_write() {
-            #[cfg(feature = "rt")]
-            for respond_to in state.respond_to.values_mut() {
-                respond_to.remove(handle);
-            }
-            state.handles.remove(handle);
-            if let Some(delayed_remove) = state.calls.remove(handle) {
-                drop(state);
-                drop(delayed_remove);
-            } else if let Some(delayed_remove) = state.callbacks.remove(handle) {
-                drop(state);
-                drop(delayed_remove);
+    pub fn remove(handle: &CallHandle, reason: &'static str) {
+        {
+            let mut delayed_drop1 = Vec::new();
+            let mut delayed_drop2 = Vec::new();
+            let mut delayed_drop3 = Vec::new();
+
+            {
+                let mut state = BusEngine::write();
+                #[cfg(feature = "rt")]
+                state.handles.remove(handle);
+                if let Some(drop_me) = state.calls.remove(handle) {
+                    trace!(
+                        "wasm_bus_drop (handle={}, reason='{}', wapm={}, topic={})",
+                        handle.id,
+                        reason,
+                        drop_me.wapm(),
+                        drop_me.topic()
+                    );
+                    delayed_drop2.push(drop_me);
+                } else if let Some(drop_me) = state.callbacks.remove(handle) {
+                    trace!(
+                        "wasm_bus_drop (handle={}, reason='{}', topic={})",
+                        handle.id,
+                        reason,
+                        drop_me.topic()
+                    );
+                    delayed_drop3.push(drop_me);
+                } else {
+                    trace!(
+                        "wasm_bus_drop (handle={}, reason='{}', orphaned)",
+                        handle.id,
+                        reason
+                    );
+                }
+                for respond_to in state.respond_to.values_mut() {
+                    if let Some(drop_me) = respond_to.remove(handle) {
+                        delayed_drop1.push(drop_me);
+                    }
+                }
             }
         }
 
@@ -166,23 +227,28 @@ impl BusEngine {
         parent: Option<CallHandle>,
         wapm: Cow<'static, str>,
         topic: Cow<'static, str>,
+        format: SerializationFormat,
+        session: Option<String>,
     ) -> Call {
-        let mut handle = CallHandle {
-            id: crate::abi::syscall::rand(),
-        };
+        use std::sync::atomic::AtomicBool;
+
+        let mut handle: CallHandle = crate::abi::syscall::handle().into();
         let mut call = Call {
+            state: Arc::new(Mutex::new(CallState {
+                result: None,
+                callbacks: Vec::new(),
+            })),
             handle,
             parent,
             wapm,
             topic,
-            state: Arc::new(Mutex::new(CallState { result: None })),
-            callbacks: Arc::new(Mutex::new(Vec::new())),
+            format,
+            session,
+            drop_on_data: Arc::new(AtomicBool::new(true)),
         };
 
         loop {
-            handle = CallHandle {
-                id: crate::abi::syscall::rand(),
-            };
+            handle = crate::abi::syscall::handle().into();
             call.handle = handle;
 
             {
@@ -204,34 +270,45 @@ impl BusEngine {
         _parent: Option<CallHandle>,
         _wapm: Cow<'static, str>,
         _topic: Cow<'static, str>,
+        _format: SerializationFormat,
+        _session: Option<String>,
     ) -> Call {
         panic!("call not supported on this platform");
     }
 
     #[cfg(target_arch = "wasm32")]
-    pub fn callback<RES, REQ, F>(mut callback: F) -> Finish
+    pub fn callback<RES, REQ, F>(format: SerializationFormat, mut callback: F) -> Finish
     where
         REQ: de::DeserializeOwned + Send + Sync + 'static,
         RES: Serialize + Send + Sync + 'static,
         F: FnMut(REQ) -> Result<RES, CallError>,
         F: Send + 'static,
     {
+        let topic = type_name::<REQ>();
         let callback = move |req: Vec<u8>| {
-            let req = bincode::deserialize::<REQ>(req.as_ref())
-                .map_err(|_err| CallError::DeserializationFailed)?;
+            let req = match format {
+                SerializationFormat::Bincode => bincode::deserialize::<REQ>(req.as_ref())
+                    .map_err(|_err| CallError::DeserializationFailed)?,
+                SerializationFormat::Json => serde_json::from_slice::<REQ>(req.as_ref())
+                    .map_err(|_err| CallError::DeserializationFailed)?,
+            };
 
             let res = callback(req)?;
 
-            let res =
-                bincode::serialize::<RES>(&res).map_err(|_err| CallError::SerializationFailed)?;
+            let res = match format {
+                SerializationFormat::Bincode => bincode::serialize::<RES>(&res)
+                    .map_err(|_err| CallError::SerializationFailed)?,
+                SerializationFormat::Json => serde_json::to_vec::<RES>(&res)
+                    .map_err(|_err| CallError::SerializationFailed)?,
+            };
 
             Ok(res)
         };
-        BusEngine::register(callback)
+        BusEngine::register(topic.into(), callback)
     }
 
     #[cfg(not(target_arch = "wasm32"))]
-    pub fn callback<RES, REQ, F>(_callback: F) -> Finish
+    pub fn callback<RES, REQ, F>(_format: SerializationFormat, _callback: F) -> Finish
     where
         REQ: de::DeserializeOwned + Send + Sync + 'static,
         RES: Serialize + Send + Sync + 'static,
@@ -242,23 +319,20 @@ impl BusEngine {
     }
 
     #[cfg(target_arch = "wasm32")]
-    fn register<F>(callback: F) -> Finish
+    fn register<F>(topic: Cow<'static, str>, callback: F) -> Finish
     where
         F: FnMut(Vec<u8>) -> Result<Vec<u8>, CallError>,
         F: Send + 'static,
     {
-        let handle = CallHandle {
-            id: crate::abi::syscall::rand(),
-        };
+        let mut handle: CallHandle = crate::abi::syscall::handle().into();
         let mut recv = Finish {
             handle: handle,
+            topic: topic.clone(),
             callback: Arc::new(Mutex::new(Box::new(callback))),
         };
 
         loop {
-            let handle = CallHandle {
-                id: crate::abi::syscall::rand(),
-            };
+            handle = crate::abi::syscall::handle().into();
             recv.handle = handle;
 
             {
@@ -276,7 +350,7 @@ impl BusEngine {
     }
 
     #[cfg(not(target_arch = "wasm32"))]
-    fn register<F>(_callback: F) -> Finish
+    fn register<F>(_topic: Cow<'static, str>, _callback: F) -> Finish
     where
         F: FnMut(Vec<u8>) -> Result<(), CallError>,
     {
@@ -285,8 +359,12 @@ impl BusEngine {
 
     #[cfg(feature = "rt")]
     #[cfg(target_arch = "wasm32")]
-    pub(crate) fn listen_internal<F, Fut>(topic: String, callback: F)
-    where
+    pub(crate) fn listen_internal<F, Fut>(
+        format: SerializationFormat,
+        topic: String,
+        callback: F,
+        persistent: bool,
+    ) where
         F: Fn(CallHandle, Vec<u8>) -> Result<Fut, CallError>,
         F: Send + Sync + 'static,
         Fut: Future<Output = Result<Vec<u8>, CallError>>,
@@ -296,10 +374,14 @@ impl BusEngine {
             let mut state = BusEngine::write();
             state.listening.insert(
                 topic.clone(),
-                ListenService::new(Arc::new(move |handle, req| {
-                    let res = callback(handle, req);
-                    Box::pin(async move { Ok(res?.await?) })
-                })),
+                ListenService::new(
+                    format,
+                    Arc::new(move |handle, req| {
+                        let res = callback(handle, req);
+                        Box::pin(async move { Ok(res?.await?) })
+                    }),
+                    persistent,
+                ),
             );
         }
 
@@ -308,8 +390,12 @@ impl BusEngine {
 
     #[cfg(feature = "rt")]
     #[cfg(not(target_arch = "wasm32"))]
-    pub(crate) fn listen_internal<F, Fut>(_topic: String, _callback: F)
-    where
+    pub(crate) fn listen_internal<F, Fut>(
+        _format: SerializationFormat,
+        _topic: String,
+        _callback: F,
+        _persistent: bool,
+    ) where
         F: Fn(CallHandle, Vec<u8>) -> Result<Fut, CallError>,
         F: Send + Sync + 'static,
         Fut: Future<Output = Result<Vec<u8>, CallError>>,
@@ -320,8 +406,13 @@ impl BusEngine {
 
     #[cfg(feature = "rt")]
     #[cfg(target_arch = "wasm32")]
-    pub(crate) fn respond_to_internal<F, Fut>(topic: String, parent: CallHandle, callback: F)
-    where
+    pub(crate) fn respond_to_internal<F, Fut>(
+        format: SerializationFormat,
+        topic: String,
+        parent: CallHandle,
+        callback: F,
+        persistent: bool,
+    ) where
         F: Fn(CallHandle, Vec<u8>) -> Result<Fut, CallError>,
         F: Send + Sync + 'static,
         Fut: Future<Output = Result<Vec<u8>, CallError>>,
@@ -332,7 +423,7 @@ impl BusEngine {
             if state.respond_to.contains_key(&topic) == false {
                 state
                     .respond_to
-                    .insert(topic.clone(), RespondToService::default());
+                    .insert(topic.clone(), RespondToService::new(format, persistent));
                 crate::abi::syscall::listen(topic.as_str());
             }
             let respond_to = state.respond_to.get_mut(&topic).unwrap();
@@ -348,8 +439,13 @@ impl BusEngine {
 
     #[cfg(feature = "rt")]
     #[cfg(not(target_arch = "wasm32"))]
-    pub(crate) fn respond_to_internal<F, Fut>(_topic: String, _parent: CallHandle, _callback: F)
-    where
+    pub(crate) fn respond_to_internal<F, Fut>(
+        _format: SerializationFormat,
+        _topic: String,
+        _parent: CallHandle,
+        _callback: F,
+        _persistent: bool,
+    ) where
         F: Fn(CallHandle, Vec<u8>) -> Result<Fut, CallError>,
         F: Send + Sync + 'static,
         Fut: Future<Output = Result<Vec<u8>, CallError>>,
